@@ -1,7 +1,7 @@
 package urpm::orphans;
 
 use strict;
-use urpm::util;
+use urpm::util qw(add2hash_ append_to_file cat_ output_safe partition put_in_hash uniq wc_l);
 use urpm::msg;
 use urpm;
 
@@ -9,6 +9,19 @@ use urpm;
 
 my $fullname2name_re = qr/^(.*)-[^\-]*-[^\-]*\.[^\.\-]*$/;
 
+
+=head1 NAME
+
+urpm::orphans - The orphan management code for urpmi
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+=over
+
+=cut
+
 #- side-effects: none
 sub installed_packages_packed {
     my ($urpm) = @_;
@@ -23,11 +36,19 @@ sub installed_packages_packed {
     \@l;
 }
 
+
+=item unrequested_list__file($urpm)
+
+Return the path of the unrequested list file.
+
+=cut
+
 #- side-effects: none
 sub unrequested_list__file {
     my ($urpm) = @_;
     ($urpm->{env_dir} || "$urpm->{root}/var/lib/rpm") . '/installed-through-deps.list';
 }
+
 #- side-effects: none
 sub unrequested_list {
     my ($urpm) = @_;
@@ -38,6 +59,13 @@ sub unrequested_list {
     } cat_(unrequested_list__file($urpm)) };
 }
 
+=item mark_as_requested($urpm, $state, $test)
+
+Mark some packages as explicitly requested (usually because
+they were manually installed).
+
+=cut
+
 #- side-effects: those of _write_unrequested_list__file
 sub mark_as_requested {
     my ($urpm, $state, $test) = @_;
@@ -68,6 +96,7 @@ sub _installed_req_and_unreq {
     my ($req, $unreq, $_unrequested) = _installed_req_and_unreq_and_update_unrequested_list($urpm);
     ($req, $unreq);
 }
+
 #- side-effects:
 #-   + those of _installed_req_and_unreq_and_update_unrequested_list (<root>/var/lib/rpm/installed-through-deps.list)
 sub _installed_and_unrequested_lists {
@@ -80,6 +109,7 @@ sub _installed_and_unrequested_lists {
 #- side-effects: <root>/var/lib/rpm/installed-through-deps.list
 sub _write_unrequested_list__file {
     my ($urpm, $unreq) = @_;
+    return if $>;
 
     $urpm->{info}("writing " . unrequested_list__file($urpm));
     
@@ -99,7 +129,11 @@ sub _installed_req_and_unreq_and_update_unrequested_list {
     my ($unreq, $req) = partition { $unrequested->{$_->name} } @$pkgs;
     
     # update the list (to filter dups and now-removed-pkgs)
-    _write_unrequested_list__file($urpm, [map { $_->name } @$unreq]);
+    my @old = keys %$unrequested;
+    my @new = map { $_->name } @$unreq;
+    if (@new != @old) {
+        _write_unrequested_list__file($urpm, \@new);
+    }
 
     ($req, $unreq, $unrequested);
 }
@@ -152,6 +186,7 @@ sub _renamed_unrequested {
     }
     %l;
 }
+
 sub new_unrequested {
     my ($urpm, $state) = @_;
     (
@@ -159,6 +194,7 @@ sub new_unrequested {
 	_renamed_unrequested($urpm, $state->{selected}, $state->{rejected}),
     );
 }
+
 #- side-effects: <root>/var/lib/rpm/installed-through-deps.list
 sub add_unrequested {
     my ($urpm, $state) = @_;
@@ -167,9 +203,15 @@ sub add_unrequested {
     append_to_file(unrequested_list__file($urpm), join('', map { "$_\t\t$l{$_}\n" } keys %l));
 }
 
-#- we don't want to check orphans on every auto-select,
-#- doing it only after many packages have been added
-#-
+=item check_unrequested_orphans_after_auto_select($urpm)
+
+We don't want to check orphans on every auto-select.
+We do it only after many packages have been added.
+
+Returns whether we should look for orphans depending on a threshold.
+
+=cut
+
 #- side-effects: none
 sub check_unrequested_orphans_after_auto_select {
     my ($urpm) = @_;
@@ -178,13 +220,20 @@ sub check_unrequested_orphans_after_auto_select {
     $nb_added >= $urpm->{options}{'nb-of-new-unrequested-pkgs-between-auto-select-orphans-check'};
 }
 
-#- this function computes wether removing $toremove packages will create
-#- unrequested orphans.
-#-
-#- it does not return the new orphans since "whatsuggests" is not available,
-#- if it detects there are new orphans, _all_unrequested_orphans()
-#- must be used to have the list of the orphans
-#-
+
+=item unrequested_orphans_after_remove($urpm, $toremove)
+
+This function computes wether removing $toremove packages will create
+unrequested orphans.
+
+It does not return the new orphans since "whatsuggests" is not
+available,
+
+If it detects there are new orphans, _all_unrequested_orphans() must
+be used to have the list of the orphans
+
+=cut
+
 #- side-effects: none
 sub unrequested_orphans_after_remove {
     my ($urpm, $toremove) = @_;
@@ -193,6 +242,7 @@ sub unrequested_orphans_after_remove {
     my %toremove = map { $_ => 1 } @$toremove;
     _unrequested_orphans_after_remove_once($urpm, $db, unrequested_list($urpm), \%toremove);
 }
+
 #- side-effects: none
 sub _unrequested_orphans_after_remove_once {
     my ($urpm, $db, $unrequested, $toremove) = @_;
@@ -228,6 +278,7 @@ sub _unrequested_orphans_after_remove_once {
     }
     0;
 }
+
 #- return true if $pkg will no more be required after removing $toremove
 #-
 #- nb: it may wrongly return false for complex loops,
@@ -296,7 +347,7 @@ sub _will_prop_still_be_needed {
 sub _get_current_kernel_package() {
     my $release = (POSIX::uname())[2];
     # --qf '%{name}' is used in order to provide the right format:
-    -e "/boot/vmlinuz-$release" && `rpm -qf --qf '%{name}' /boot/vmlinuz-$release`;
+    -e "/boot/vmlinuz-$release" && ($release, `rpm -qf --qf '%{name}' /boot/vmlinuz-$release`);
 }
 
 
@@ -312,7 +363,7 @@ sub _get_current_kernel_package() {
 # do not care about (eg: kernel-devel, kernel-firmware, kernel-latest)
 # so it's useless to look at them
 #
-my (@latest_kernels, %requested_kernels, %kernels);
+my (@req_by_latest_kernels, %requested_kernels, %kernels);
 sub _kernel_callback { 
     my ($pkg, $unreq_list) = @_;
     my $shortname = $pkg->name;
@@ -321,7 +372,7 @@ sub _kernel_callback {
     # only consider kernels (and not main 'kernel' package):
     # but perform a pass on their requires for dkms like packages that require a specific kernel:
     if ($shortname !~ /^kernel-/) {
-	foreach (grep { /^kernel/ } $pkg->requires) {
+	foreach (grep { /^kernel/ } $pkg->requires_nosense) {
 	    $requested_kernels{$_}{$shortname} = $pkg;
 	}
 	return;
@@ -333,9 +384,9 @@ sub _kernel_callback {
     # ignore requested kernels (aka that are not in /var/lib/rpm/installed-through-deps.list)
     return if !$unreq_list->{$shortname} && $shortname !~ /latest/;
 
-    # keep track of latest kernels in order not to try removing requested kernels:
+    # keep track of packages required by latest kernels in order not to try removing requested kernels:
     if ($n =~ /latest/) {
-        push @latest_kernels, $pkg->requires;
+        push @req_by_latest_kernels, $pkg->requires;
     } else {
         $kernels{$shortname} = $pkg;
     }
@@ -345,9 +396,9 @@ sub _kernel_callback {
 # - returns list of orphan kernels
 sub _get_orphan_kernels() {
     # keep kernels required by kernel-*-latest:
-    delete $kernels{$_} foreach @latest_kernels;
+    delete $kernels{$_} foreach @req_by_latest_kernels;
     # return list of unused/orphan kernels:
-    %kernels;
+    \%kernels;
 }
 
 
@@ -364,7 +415,7 @@ sub _all_unrequested_orphans {
     }
     my $unreq_list = unrequested_list($urpm);
 
-    my $current_kernel = _get_current_kernel_package();
+    my ($current_kernel_version, $current_kernel) = _get_current_kernel_package();
 
     while (my $pkg = shift @$req) {
         # do not do anything regarding kernels if we failed to detect the running one (ie: chroot)
@@ -381,7 +432,7 @@ sub _all_unrequested_orphans {
     }
 
     # add orphan kernels to the list:
-    my $a = { _get_orphan_kernels() };
+    my $a = _get_orphan_kernels();
     add2hash_(\%l, $a);
 
     # add packages that require orphan kernels to the list:
@@ -389,11 +440,18 @@ sub _all_unrequested_orphans {
 	add2hash_(\%l, $requested_kernels{$_});
     }
 
-    # do not offer to remove current kernel:
+    # do not offer to remove current kernel or DKMS modules for current kernel:
     delete $l{$current_kernel};
+    do { delete $l{$_} } foreach grep { /$current_kernel_version/ } keys %l;
     [ values %l ];
 }
 
+=item compute_future_unrequested_orphans($urpm, $state)
+
+Compute the list of packages that will be unrequested and
+could potently be removed.
+
+=cut
 
 #- side-effects: $state->{orphans_to_remove}
 #-   + those of _installed_and_unrequested_lists (<root>/var/lib/rpm/installed-through-deps.list)
@@ -417,8 +475,16 @@ sub compute_future_unrequested_orphans {
     # nb: $state->{orphans_to_remove} is used when computing ->selected_size
 }
 
-#- it is quite fast. the slow part is the creation of $installed_packages_packed
-#- (using installed_packages_packed())
+
+=item get_orphans($urpm)
+
+Returns the list of unrequested packages (aka orphans).
+
+It is quite fast. the slow part is the creation of
+$installed_packages_packed (using installed_packages_packed())
+
+=cut
+
 #
 #- side-effects:
 #-   + those of _installed_req_and_unreq (<root>/var/lib/rpm/installed-through-deps.list)
@@ -440,6 +506,15 @@ sub _get_now_orphans_raw_msg {
     (scalar(@orphans), add_leading_spaces(join("\n", sort @orphans)));
 }
 
+=item get_now_orphans_gui_msg($urpm)
+
+Like get_now_orphans_msg() but more suited for GUIes, it return
+message about orphan packages.
+
+Used by rpmdrake.
+
+=cut
+
 sub get_now_orphans_gui_msg {
     my ($urpm) = @_;
 
@@ -453,6 +528,14 @@ sub get_now_orphans_gui_msg {
     );
 }
 
+
+=item get_now_orphans_msg($urpm)
+
+Similar to get_now_orphans_gui_msg() but more suited for CLI, it
+return message about orphan packages.
+
+=cut
+
 sub get_now_orphans_msg {
     my ($urpm) = @_;
 
@@ -463,6 +546,12 @@ sub get_now_orphans_msg {
 }
 
 
+=item add_leading_spaces($string)
+
+Add leading spaces to the string and return it.
+
+=cut
+
 #- side-effects: none
 sub add_leading_spaces {
     my ($s) = @_;
@@ -497,3 +586,15 @@ sub installed_leaves {
 }
 
 1;
+
+
+=back
+
+=head1 COPYRIGHT
+
+
+Copyright (C) 2008-2010 Mandriva SA
+
+Copyright (C) 2011-2013 Mageia
+
+=cut
